/**
 * Service
 * @class ApToastService
 * @param {Object} store - Redux store
 * @param {Object} options - Optional settings
 */
'use strict'

const { ApService } = require('apeman-service-base')
const abind = require('abind')
const co = require('co')
const asleep = require('asleep')

const INFO = 'info'
const WARN = 'warn'
const ERROR = 'error'

/** @lends ApToastService */
class ApToastService extends ApService {
  constructor (store, options = {}) {
    super(store)
    const s = this
    abind(s, { proto: ApToastService.prototype })
    abind(s)
    let {
      interval = 2000
    } = options
    s.interval = interval
  }

  // ---------------
  // Getter Methods
  // ---------------

  /**
   * Get toast
   * @param {string} level - Toast level
   * @returns {Array.<string>}
   */
  getToasts (level) {
    const s = this
    return (s.store.get(level) || [])
      .filter((message, i, array) => array.indexOf(message) === i)
  }

  // ---------------
  // Action Methods
  // ---------------

  /**
   * Show toast
   * @param {string} level - Toast level
   * @param {string} text - Toast text
   * @returns {Promise}
   */
  toast (level, text) {
    const s = this
    let { delay, interval } = s
    return co(function * () {
      yield asleep(delay)
      s.store.push(level, text)
      yield asleep(interval)
      s.store.reject(level, text)
    })
  }

  /**
   * Show info toast
   * @param text
   * @returns {Promise}
   */
  infoToast (text) {
    const s = this
    return s.toast(INFO, text)
  }

  /**
   * Show warn toast
   * @param text
   * @returns {Promise}
   */
  warnToast (text) {
    const s = this
    return s.toast(WARN, text)
  }

  /**
   * Show error toast
   * @param text
   * @returns {Promise}
   */
  errorToast (text) {
    const s = this
    return s.toast(ERROR, text)
  }
}

Object.assign(ApToastService, { INFO, WARN, ERROR })

module.exports = ApToastService
